@@ -99,6 +99,16 @@ def mode(ctx, mode, touch_eject, autoeject_timeout, chalresp_timeout, force):
 
     MODE can be a string, such as "OTP+FIDO+CCID", or a shortened form: "o+f+c".
     It can also be a mode number.
+
+    Examples:
+
+    \b
+      Set the OTP and FIDO mode:
+      $ ykman mode OTP+FIDO
+
+    \b
+      Set the CCID only mode and use touch to eject the smart card:
+      $ ykman mode CCID --touch-eject
     """
     dev = ctx.obj['dev']
     if autoeject_timeout:
@@ -121,7 +131,7 @@ def mode(ctx, mode, touch_eject, autoeject_timeout, chalresp_timeout, force):
                            .format(mode))
                 ctx.fail('Use --force to attempt to set it anyway.')
             force or click.confirm('Set mode of YubiKey to {}?'.format(mode),
-                                   abort=True)
+                                   abort=True, err=True)
 
         try:
             dev.set_mode(mode, chalresp_timeout, autoeject)
