@@ -39,7 +39,7 @@ from ..driver_ccid import APDUError, SW
 from .util import (
     click_force_option, click_format_option,
     click_postpone_execution, click_callback,
-    prompt_for_touch, UpperCaseChoice)
+    prompt_for_touch, EnumChoice)
 from cryptography import x509
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.backends import default_backend
@@ -81,11 +81,10 @@ click_pin_option = click.option(
     '-P', '--pin', help='PIN code.')
 click_pin_policy_option = click.option(
     '--pin-policy',
-    type=UpperCaseChoice(['DEFAULT', 'NEVER', 'ONCE', 'ALWAYS']),
+    type=EnumChoice(PIN_POLICY),
     help='PIN policy for slot.')
 click_touch_policy_option = click.option(
-    '--touch-policy', type=UpperCaseChoice(
-        ['DEFAULT', 'NEVER', 'ALWAYS', 'CACHED']),
+    '--touch-policy', type=EnumChoice(TOUCH_POLICY),
     help='Touch policy for slot.')
 
 
@@ -180,9 +179,16 @@ def info(ctx):
         fingerprint = b2a_hex(cert.fingerprint(hashes.SHA256())).decode('ascii')
         algo = ALGO.from_public_key(cert.public_key())
         serial = cert.serial_number
-        not_before = cert.not_valid_before
-        not_after = cert.not_valid_after
-
+        try:
+            not_before = cert.not_valid_before
+        except ValueError as e:
+            logger.debug('Failed reading not_valid_before', exc_info=e)
+            not_before = None
+        try:
+            not_after = cert.not_valid_after
+        except ValueError as e:
+            logger.debug('Failed reading not_valid_after', exc_info=e)
+            not_after = None
         # Print out everything
         click.echo('\tAlgorithm:\t%s' % algo.name)
         if print_dn:
@@ -193,8 +199,10 @@ def info(ctx):
             click.echo('\tIssuer CN:\t%s' % issuer_cn)
         click.echo('\tSerial:\t\t%s' % serial)
         click.echo('\tFingerprint:\t%s' % fingerprint)
-        click.echo('\tNot before:\t%s' % not_before)
-        click.echo('\tNot after:\t%s' % not_after)
+        if not_before:
+            click.echo('\tNot before:\t%s' % not_before)
+        if not_after:
+            click.echo('\tNot after:\t%s' % not_after)
 
 
 @piv.command()
@@ -228,8 +236,7 @@ def reset(ctx):
 @click_pin_option
 @click.option(
     '-a', '--algorithm', help='Algorithm to use in key generation.',
-    type=UpperCaseChoice(
-        ['RSA1024', 'RSA2048', 'ECCP256', 'ECCP384']), default='RSA2048',
+    type=EnumChoice(ALGO), default=ALGO.RSA2048.name,
     show_default=True)
 @click_format_option
 @click_pin_policy_option
@@ -255,25 +262,18 @@ def generate_key(
 
     _ensure_authenticated(ctx, controller, pin, management_key)
 
-    algorithm_id = ALGO.from_string(algorithm)
-
-    if pin_policy:
-        pin_policy = PIN_POLICY.from_string(pin_policy)
-    if touch_policy:
-        touch_policy = TOUCH_POLICY.from_string(touch_policy)
-
     _check_pin_policy(ctx, dev, controller, pin_policy)
     _check_touch_policy(ctx, controller, touch_policy)
 
     try:
         public_key = controller.generate_key(
             slot,
-            algorithm_id,
+            algorithm,
             pin_policy,
             touch_policy)
     except UnsupportedAlgorithm:
         ctx.fail('Algorithm {} is not supported by this '
-                 'YubiKey.'.format(algorithm))
+                 'YubiKey.'.format(algorithm.name))
 
     key_encoding = format
     public_key_output.write(public_key.public_bytes(
@@ -402,11 +402,6 @@ def import_key(
             continue
         break
 
-    if pin_policy:
-        pin_policy = PIN_POLICY.from_string(pin_policy)
-    if touch_policy:
-        touch_policy = TOUCH_POLICY.from_string(touch_policy)
-
     _check_pin_policy(ctx, dev, controller, pin_policy)
     _check_touch_policy(ctx, controller, touch_policy)
     _check_key_size(ctx, controller, private_key)
@@ -561,7 +556,7 @@ def generate_certificate(
     public_key = serialization.load_pem_public_key(
         data, default_backend())
 
-    now = datetime.datetime.now()
+    now = datetime.datetime.utcnow()
     valid_to = now + datetime.timedelta(days=valid_days)
 
     try:
