@@ -98,7 +98,7 @@ class PrepareUploadError(Enum):
 
 class PrepareUploadFailed(Exception):
     def __init__(self, status, content, error_ids):
-        super().__init__(
+        super(PrepareUploadFailed, self).__init__(
             'Upload to YubiCloud failed with status {}: {}'
             .format(status, content))
         self.status = status
