@@ -31,7 +31,7 @@ from enum import IntEnum, unique
 from .device import YubiKey
 from .driver_ccid import APDUError, SW
 from .util import (
-    AID, Tlv, parse_tlvs,
+    AID, Tlv,
     is_cve201715361_vulnerable_firmware_version,
     ensure_not_cve201715361_vulnerable_firmware_version)
 from cryptography import x509
@@ -95,20 +95,6 @@ class ALGO(IntEnum):
         raise UnsupportedAlgorithm(
             'Unsupported key type: %s' % type(key), key=key)
 
-    @classmethod
-    def from_string(cls, algorithm):
-        if algorithm == 'RSA1024':
-            return cls.RSA1024
-        if algorithm == 'RSA2048':
-            return cls.RSA2048
-        if algorithm == 'ECCP256':
-            return cls.ECCP256
-        if algorithm == 'ECCP384':
-            return cls.ECCP384
-        raise UnsupportedAlgorithm(
-            'Unsupported algorithm name: %s' % algorithm,
-            algorithm_id=algorithm)
-
     @classmethod
     def is_rsa(cls, algorithm_int):
         # Implemented as "not not RSA" to reduce risk of false negatives if
@@ -217,18 +203,6 @@ class PIN_POLICY(IntEnum):
     ONCE = 0x2
     ALWAYS = 0x3
 
-    @classmethod
-    def from_string(cls, pin_policy):
-        if pin_policy == 'DEFAULT':
-            return cls.DEFAULT
-        if pin_policy == 'NEVER':
-            return cls.NEVER
-        if pin_policy == 'ONCE':
-            return cls.ONCE
-        if pin_policy == 'ALWAYS':
-            return cls.ALWAYS
-        raise UnknownPinPolicy(pin_policy)
-
 
 @unique
 class TOUCH_POLICY(IntEnum):
@@ -237,22 +211,10 @@ class TOUCH_POLICY(IntEnum):
     ALWAYS = 0x2
     CACHED = 0x3
 
-    @classmethod
-    def from_string(cls, touch_policy):
-        if touch_policy == 'DEFAULT':
-            return cls.DEFAULT
-        if touch_policy == 'NEVER':
-            return cls.NEVER
-        if touch_policy == 'ALWAYS':
-            return cls.ALWAYS
-        if touch_policy == 'CACHED':
-            return cls.CACHED
-        raise UnknownTouchPolicy(touch_policy)
-
 
 class AuthenticationFailed(Exception):
     def __init__(self, message, sw, applet_version):
-        super().__init__(message)
+        super(AuthenticationFailed, self).__init__(message)
         self.tries_left = (
             tries_left(sw, applet_version)
             if is_verify_fail(sw, applet_version)
@@ -262,18 +224,18 @@ class AuthenticationFailed(Exception):
 class AuthenticationBlocked(AuthenticationFailed):
     def __init__(self, message, sw):
         # Dummy applet_version since sw will always be "authentication blocked"
-        super().__init__(message, sw, ())
+        super(AuthenticationBlocked, self).__init__(message, sw, ())
 
 
 class BadFormat(Exception):
     def __init__(self, message, bad_value):
-        super().__init__(message)
+        super(BadFormat, self).__init__(message)
         self.bad_value = bad_value
 
 
 class KeypairMismatch(Exception):
     def __init__(self, slot, cert):
-        super().__init__(
+        super(KeypairMismatch, self).__init__(
             'The certificate does not match the private key in slot %s.' % slot)
         self.slot = slot
         self.cert = cert
@@ -281,7 +243,7 @@ class KeypairMismatch(Exception):
 
 class UnsupportedAlgorithm(Exception):
     def __init__(self, message, algorithm_id=None, key=None, ):
-        super().__init__(message)
+        super(UnsupportedAlgorithm, self).__init__(message)
         if algorithm_id is None and key is None:
             raise ValueError(
                 'At least one of algorithm_id and key must be given.')
@@ -290,28 +252,16 @@ class UnsupportedAlgorithm(Exception):
         self.key = key
 
 
-class UnknownPinPolicy(Exception):
-    def __init__(self, policy_name):
-        super().__init__(
-            'Unsupported pin policy: %s' % policy_name)
-        self.policy_name = policy_name
-
-
-class UnknownTouchPolicy(Exception):
-    def __init__(self, policy_name):
-        super().__init__(
-            'Unsupported touch policy: %s' % policy_name)
-        self.policy_name = policy_name
-
-
 class WrongPin(AuthenticationFailed):
     def __init__(self, sw, applet_version):
-        super().__init__('Incorrect PIN', sw, applet_version)
+        super(WrongPin, self).__init__(
+            'Incorrect PIN', sw, applet_version)
 
 
 class WrongPuk(AuthenticationFailed):
     def __init__(self, sw, applet_version):
-        super().__init__('Incorrect PUK', sw, applet_version)
+        super(WrongPuk, self).__init__(
+            'Incorrect PUK', sw, applet_version)
 
 
 PIN = 0x80
@@ -323,10 +273,6 @@ DEFAULT_MANAGEMENT_KEY = b'\x01\x02\x03\x04\x05\x06\x07\x08' \
     + b'\x01\x02\x03\x04\x05\x06\x07\x08'
 
 
-def _parse_tlv_dict(data):
-    return dict((tlv.tag, tlv.value) for tlv in parse_tlvs(data))
-
-
 def _pack_pin(pin):
     if isinstance(pin, six.text_type):
         pin = pin.encode('utf8')
@@ -449,7 +395,7 @@ def tries_left(sw, applet_version):
 class PivmanData(object):
 
     def __init__(self, raw_data=Tlv(0x80)):
-        data = _parse_tlv_dict(Tlv(raw_data).value)
+        data = Tlv.parse_dict(Tlv(raw_data).value)
         self._flags = struct.unpack(
             '>B', data[0x81])[0] if 0x81 in data else None
         self.salt = data.get(0x82)
@@ -495,7 +441,7 @@ class PivmanData(object):
 class PivmanProtectedData(object):
 
     def __init__(self, raw_data=Tlv(0x88)):
-        data = _parse_tlv_dict(Tlv(raw_data).value)
+        data = Tlv.parse_dict(Tlv(raw_data).value)
         self.key = data.get(0x89)
 
     def get_bytes(self):
@@ -838,11 +784,11 @@ class PivController(object):
             data += Tlv(TAG.TOUCH_POLICY, six.int2byte(touch_policy))
         data = Tlv(0xac, data)
         resp = self.send_cmd(INS.GENERATE_ASYMMETRIC, 0, slot, data)
+        key_data = Tlv.parse_dict(Tlv.unpack(0x7f49, resp))
         if algorithm in [ALGO.RSA1024, ALGO.RSA2048]:
-            data = _parse_tlv_dict(Tlv(resp[1:]).value)
             return rsa.RSAPublicNumbers(
-                int_from_bytes(data[0x82], 'big'),
-                int_from_bytes(data[0x81], 'big')
+                int_from_bytes(key_data[0x82], 'big'),
+                int_from_bytes(key_data[0x81], 'big')
             ).public_key(default_backend())
         elif algorithm in [ALGO.ECCP256, ALGO.ECCP384]:
             curve = ec.SECP256R1 if algorithm == ALGO.ECCP256 else ec.SECP384R1
@@ -851,12 +797,12 @@ class PivController(object):
                 # Added in cryptography 2.5
                 return ec.EllipticCurvePublicKey.from_encoded_point(
                     curve(),
-                    resp[5:]
+                    key_data[0x86]
                 )
             except AttributeError:
                 return ec.EllipticCurvePublicNumbers.from_encoded_point(
                     curve(),
-                    resp[5:]
+                    key_data[0x86]
                 ).public_key(default_backend())
 
         raise UnsupportedAlgorithm(
@@ -965,7 +911,7 @@ class PivController(object):
         self.update_chuid()
 
     def read_certificate(self, slot):
-        data = _parse_tlv_dict(self.get_data(OBJ.from_slot(slot)))
+        data = Tlv.parse_dict(self.get_data(OBJ.from_slot(slot)))
         if TAG.CERT_INFO in data:  # Not available in attestation slot
             if data[TAG.CERT_INFO] != b'\0':
                 raise ValueError('Compressed certificates are not supported!')
@@ -987,7 +933,7 @@ class PivController(object):
 
         data = Tlv(TAG.DYN_AUTH, Tlv(0x82) + payload)
         resp = self.send_cmd(INS.AUTHENTICATE, algorithm, slot, data)
-        return Tlv(Tlv(resp).value).value
+        return Tlv.unpack(0x82, Tlv.unpack(0x7c, resp))
 
     def sign_raw(self, slot, algorithm, message):
         return self._raw_sign_decrypt(slot, algorithm, Tlv(0x81, message),
@@ -1063,7 +1009,7 @@ class PivController(object):
         if touch_callback is not None:
             touch_timer.cancel()
 
-        seq = parse_tlvs(Tlv(cert.public_bytes(Encoding.DER)).value)
+        seq = Tlv.parse_list(Tlv.unpack(0x30, cert.public_bytes(Encoding.DER)))
         # Replace signature, add unused bits = 0
         seq[2] = Tlv(seq[2].tag, b'\0' + sig)
         # Re-assemble sequence
@@ -1075,7 +1021,7 @@ class PivController(object):
         algorithm = ALGO.from_public_key(public_key)
         dummy_key = _dummy_key(algorithm)
         csr = builder.sign(dummy_key, hashes.SHA256(), default_backend())
-        seq = parse_tlvs(Tlv(csr.public_bytes(Encoding.DER)).value)
+        seq = Tlv.parse_list(Tlv.unpack(0x30, csr.public_bytes(Encoding.DER)))
 
         # Replace public key
         pub_format = PublicFormat.PKCS1 if algorithm.name.startswith('RSA') \
